@@ -30,11 +30,13 @@ states, visit :ref:`the Audit State machine <audit_state_machine>`.
 """
 
 import datetime
+from dateutil import tz
 
 import pecan
 from pecan import rest
 import wsme
 from wsme import types as wtypes
+from wsme import utils as wutils
 import wsmeext.pecan as wsme_pecan
 
 from oslo_log import log
@@ -54,6 +56,25 @@ from watcher import objects
 LOG = log.getLogger(__name__)
 
 
+def _get_object_by_value(context, class_name, value):
+    if utils.is_uuid_like(value) or utils.is_int_like(value):
+        return class_name.get(context, value)
+    else:
+        return class_name.get_by_name(context, value)
+
+
+def hide_fields_in_newer_versions(obj):
+    """This method hides fields that were added in newer API versions.
+
+    Certain node fields were introduced at certain API versions.
+    These fields are only made available when the request's API version
+    matches or exceeds the versions when these fields were introduced.
+    """
+    if not api_utils.allow_start_end_audit_time():
+        obj.start_time = wsme.Unset
+        obj.end_time = wsme.Unset
+
+
 class AuditPostType(wtypes.Base):
 
     name = wtypes.wsattr(wtypes.text, mandatory=False)
@@ -77,6 +98,12 @@ class AuditPostType(wtypes.Base):
 
     auto_trigger = wtypes.wsattr(bool, mandatory=False)
 
+    hostname = wtypes.wsattr(wtypes.text, readonly=True, mandatory=False)
+
+    start_time = wsme.wsattr(datetime.datetime, mandatory=False)
+
+    end_time = wsme.wsattr(datetime.datetime, mandatory=False)
+
     def as_audit(self, context):
         audit_type_values = [val.value for val in objects.audit.AuditType]
         if self.audit_type not in audit_type_values:
@@ -91,6 +118,21 @@ class AuditPostType(wtypes.Base):
             raise exception.AuditIntervalNotSpecified(
                 audit_type=self.audit_type)
 
+        if self.audit_template_uuid and self.goal:
+            raise exception.Invalid('Either audit_template_uuid '
+                                    'or goal should be provided.')
+
+        if (self.audit_type == objects.audit.AuditType.ONESHOT.value and
+                (self.start_time not in (wtypes.Unset, None) or
+                    self.end_time not in (wtypes.Unset, None))):
+            raise exception.AuditStartEndTimeNotAllowed(
+                audit_type=self.audit_type)
+
+        if not api_utils.allow_start_end_audit_time():
+            for field in ('start_time', 'end_time'):
+                if getattr(self, field) not in (wsme.Unset, None):
+                    raise exception.NotAcceptable()
+
         # If audit_template_uuid was provided, we will provide any
         # variables not included in the request, but not override
         # those variables that were included.
@@ -121,7 +163,8 @@ class AuditPostType(wtypes.Base):
         # Note: If audit name was not provided, used a default name
         if not self.name:
             if self.strategy:
-                strategy = objects.Strategy.get(context, self.strategy)
+                strategy = _get_object_by_value(context, objects.Strategy,
+                                                self.strategy)
                 self.name = "%s-%s" % (strategy.name,
                                        datetime.datetime.utcnow().isoformat())
             elif self.audit_template_uuid:
@@ -130,7 +173,7 @@ class AuditPostType(wtypes.Base):
                 self.name = "%s-%s" % (audit_template.name,
                                        datetime.datetime.utcnow().isoformat())
             else:
-                goal = objects.Goal.get(context, self.goal)
+                goal = _get_object_by_value(context, objects.Goal, self.goal)
                 self.name = "%s-%s" % (goal.name,
                                        datetime.datetime.utcnow().isoformat())
         # No more than 63 characters
@@ -147,7 +190,9 @@ class AuditPostType(wtypes.Base):
             strategy_id=self.strategy,
             interval=self.interval,
             scope=self.scope,
-            auto_trigger=self.auto_trigger)
+            auto_trigger=self.auto_trigger,
+            start_time=self.start_time,
+            end_time=self.end_time)
 
 
 class AuditPatchType(types.JsonPatchType):
@@ -305,6 +350,15 @@ class Audit(base.APIBase):
     next_run_time = wsme.wsattr(datetime.datetime, mandatory=False)
     """The next time audit launch"""
 
+    hostname = wsme.wsattr(wtypes.text, mandatory=False)
+    """Hostname the audit is running on"""
+
+    start_time = wsme.wsattr(datetime.datetime, mandatory=False)
+    """The start time for continuous audit launch"""
+
+    end_time = wsme.wsattr(datetime.datetime, mandatory=False)
+    """The end time that stopping continuous audit"""
+
     def __init__(self, **kwargs):
         self.fields = []
         fields = list(objects.Audit.fields)
@@ -351,6 +405,7 @@ class Audit(base.APIBase):
     @classmethod
     def convert_with_links(cls, rpc_audit, expand=True):
         audit = Audit(**rpc_audit.as_dict())
+        hide_fields_in_newer_versions(audit)
         return cls._convert_with_links(audit, pecan.request.host_url, expand)
 
     @classmethod
@@ -365,7 +420,9 @@ class Audit(base.APIBase):
                      interval='7200',
                      scope=[],
                      auto_trigger=False,
-                     next_run_time=datetime.datetime.utcnow())
+                     next_run_time=datetime.datetime.utcnow(),
+                     start_time=datetime.datetime.utcnow(),
+                     end_time=datetime.datetime.utcnow())
 
         sample.goal_id = '7ae81bb3-dec3-4289-8d6c-da80bd8001ae'
         sample.strategy_id = '7ae81bb3-dec3-4289-8d6c-da80bd8001ff'
@@ -389,17 +446,6 @@ class AuditCollection(collection.Collection):
         collection = AuditCollection()
         collection.audits = [Audit.convert_with_links(p, expand)
                              for p in rpc_audits]
-
-        if 'sort_key' in kwargs:
-            reverse = False
-            if kwargs['sort_key'] == 'goal_uuid':
-                if 'sort_dir' in kwargs:
-                    reverse = True if kwargs['sort_dir'] == 'desc' else False
-                collection.audits = sorted(
-                    collection.audits,
-                    key=lambda audit: audit.goal_uuid,
-                    reverse=reverse)
-
         collection.next = collection.get_next(limit, url=url, **kwargs)
         return collection
 
@@ -414,6 +460,7 @@ class AuditsController(rest.RestController):
     """REST controller for Audits."""
     def __init__(self):
         super(AuditsController, self).__init__()
+        self.dc_client = rpcapi.DecisionEngineAPI()
 
     from_audits = False
     """A flag to indicate if the requests to this controller are coming
@@ -427,8 +474,14 @@ class AuditsController(rest.RestController):
                                sort_key, sort_dir, expand=False,
                                resource_url=None, goal=None,
                                strategy=None):
+        additional_fields = ["goal_uuid", "goal_name", "strategy_uuid",
+                             "strategy_name"]
+
+        api_utils.validate_sort_key(
+            sort_key, list(objects.Audit.fields) + additional_fields)
         limit = api_utils.validate_limit(limit)
         api_utils.validate_sort_dir(sort_dir)
+
         marker_obj = None
         if marker:
             marker_obj = objects.Audit.get_by_uuid(pecan.request.context,
@@ -449,26 +502,28 @@ class AuditsController(rest.RestController):
                 # TODO(michaelgugino): add method to get goal by name.
                 filters['strategy_name'] = strategy
 
-        if sort_key == 'goal_uuid':
-            sort_db_key = 'goal_id'
-        elif sort_key == 'strategy_uuid':
-            sort_db_key = 'strategy_id'
-        else:
-            sort_db_key = sort_key
+        need_api_sort = api_utils.check_need_api_sort(sort_key,
+                                                      additional_fields)
+        sort_db_key = (sort_key if not need_api_sort
+                       else None)
 
         audits = objects.Audit.list(pecan.request.context,
                                     limit,
                                     marker_obj, sort_key=sort_db_key,
                                     sort_dir=sort_dir, filters=filters)
 
-        return AuditCollection.convert_with_links(audits, limit,
-                                                  url=resource_url,
-                                                  expand=expand,
-                                                  sort_key=sort_key,
-                                                  sort_dir=sort_dir)
+        audits_collection = AuditCollection.convert_with_links(
+            audits, limit, url=resource_url, expand=expand,
+            sort_key=sort_key, sort_dir=sort_dir)
+
+        if need_api_sort:
+            api_utils.make_api_sort(audits_collection.audits, sort_key,
+                                    sort_dir)
+
+        return audits_collection
 
     @wsme_pecan.wsexpose(AuditCollection, types.uuid, int, wtypes.text,
-                         wtypes.text, wtypes.text, wtypes.text, int)
+                         wtypes.text, wtypes.text, wtypes.text)
     def get_all(self, marker=None, limit=None, sort_key='id', sort_dir='asc',
                 goal=None, strategy=None):
         """Retrieve a list of audits.
@@ -569,6 +624,17 @@ class AuditsController(rest.RestController):
                                       'parameter spec in predefined strategy'))
 
         audit_dict = audit.as_dict()
+        # convert local time to UTC time
+        start_time_value = audit_dict.get('start_time')
+        end_time_value = audit_dict.get('end_time')
+        if start_time_value:
+            audit_dict['start_time'] = start_time_value.replace(
+                tzinfo=tz.tzlocal()).astimezone(
+                    tz.tzutc()).replace(tzinfo=None)
+        if end_time_value:
+            audit_dict['end_time'] = end_time_value.replace(
+                tzinfo=tz.tzlocal()).astimezone(
+                    tz.tzutc()).replace(tzinfo=None)
 
         new_audit = objects.Audit(context, **audit_dict)
         new_audit.create()
@@ -578,8 +644,7 @@ class AuditsController(rest.RestController):
 
         # trigger decision-engine to run the audit
         if new_audit.audit_type == objects.audit.AuditType.ONESHOT.value:
-            dc_client = rpcapi.DecisionEngineAPI()
-            dc_client.trigger_audit(context, new_audit.uuid)
+            self.dc_client.trigger_audit(context, new_audit.uuid)
 
         return Audit.convert_with_links(new_audit)
 
@@ -614,6 +679,16 @@ class AuditsController(rest.RestController):
                     reason=error_message % dict(
                         initial_state=initial_state, new_state=new_state))
 
+            patch_path = api_utils.get_patch_key(patch, 'path')
+            if patch_path in ('start_time', 'end_time'):
+                patch_value = api_utils.get_patch_value(patch, patch_path)
+                # convert string format to UTC time
+                new_patch_value = wutils.parse_isodatetime(
+                    patch_value).replace(
+                        tzinfo=tz.tzlocal()).astimezone(
+                            tz.tzutc()).replace(tzinfo=None)
+                api_utils.set_patch_value(patch, patch_path, new_patch_value)
+
             audit = Audit(**api_utils.apply_jsonpatch(audit_dict, patch))
         except api_utils.JSONPATCH_EXCEPTIONS as e:
             raise exception.PatchError(patch=patch, reason=e)
@@ -642,8 +717,8 @@ class AuditsController(rest.RestController):
         context = pecan.request.context
         audit_to_delete = api_utils.get_resource(
             'Audit', audit, eager=True)
-        policy.enforce(context, 'audit:update', audit_to_delete,
-                       action='audit:update')
+        policy.enforce(context, 'audit:delete', audit_to_delete,
+                       action='audit:delete')
 
         initial_state = audit_to_delete.state
         new_state = objects.audit.State.DELETED
