@@ -27,9 +27,12 @@ NEUTRON_CLIENT_OPTS = [
                help='Version of Neutron API to use in neutronclient.'),
     cfg.StrOpt('endpoint_type',
                default='publicURL',
-               help='Type of endpoint to use in neutronclient.'
-                    'Supported values: internalURL, publicURL, adminURL'
-                    'The default is publicURL.')]
+               help='Type of endpoint to use in neutronclient. '
+                    'Supported values: internalURL, publicURL, adminURL. '
+                    'The default is publicURL.'),
+    cfg.StrOpt('region_name',
+               help='Region in Identity service catalog to use for '
+                    'communication with the OpenStack service.')]
 
 
 def register_opts(conf):
