@@ -23,8 +23,10 @@ from sqlalchemy import Boolean
 from sqlalchemy import Column
 from sqlalchemy import DateTime
 from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy import Float
 from sqlalchemy import ForeignKey
 from sqlalchemy import Integer
+from sqlalchemy import LargeBinary
 from sqlalchemy import Numeric
 from sqlalchemy import orm
 from sqlalchemy import String
@@ -181,6 +183,9 @@ class Audit(Base):
     scope = Column(JSONEncodedList, nullable=True)
     auto_trigger = Column(Boolean, nullable=False)
     next_run_time = Column(DateTime, nullable=True)
+    hostname = Column(String(255), nullable=True)
+    start_time = Column(DateTime, nullable=True)
+    end_time = Column(DateTime, nullable=True)
 
     goal = orm.relationship(Goal, foreign_keys=goal_id, lazy=None)
     strategy = orm.relationship(Strategy, foreign_keys=strategy_id, lazy=None)
@@ -200,6 +205,7 @@ class ActionPlan(Base):
     strategy_id = Column(Integer, ForeignKey('strategies.id'), nullable=False)
     state = Column(String(20), nullable=True)
     global_efficacy = Column(JSONEncodedList, nullable=True)
+    hostname = Column(String(255), nullable=True)
 
     audit = orm.relationship(Audit, foreign_keys=audit_id, lazy=None)
     strategy = orm.relationship(Strategy, foreign_keys=strategy_id, lazy=None)
@@ -294,3 +300,23 @@ class ActionDescription(Base):
     id = Column(Integer, primary_key=True)
     action_type = Column(String(255), nullable=False)
     description = Column(String(255), nullable=False)
+
+
+class APScheulerJob(Base):
+    """Represents apscheduler jobs"""
+
+    __tablename__ = 'apscheduler_jobs'
+    __table_args__ = (
+        UniqueConstraint('id',
+                         name="uniq_apscheduler_jobs0id"),
+        table_args()
+    )
+    id = Column(String(191), nullable=False, primary_key=True)
+    next_run_time = Column(Float(25), index=True)
+    job_state = Column(LargeBinary, nullable=False)
+    tag = Column(JSONEncodedDict(), nullable=True)
+    service_id = Column(Integer, ForeignKey('services.id'),
+                        nullable=False)
+
+    service = orm.relationship(
+        Service, foreign_keys=service_id, lazy=None)
