@@ -58,22 +58,24 @@ class ContinuousAuditHandler(base.AuditHandler):
 
     def _is_audit_inactive(self, audit):
         audit = objects.Audit.get_by_uuid(
-            self.context_show_deleted, audit.uuid)
-        if objects.audit.AuditStateTransitionManager().is_inactive(audit):
+            self.context_show_deleted, audit.uuid, eager=True)
+        if (objects.audit.AuditStateTransitionManager().is_inactive(audit) or
+                (audit.hostname != CONF.host) or
+                (self.check_audit_expired(audit))):
             # if audit isn't in active states, audit's job must be removed to
             # prevent using of inactive audit in future.
-            if self.scheduler.get_jobs():
-                [job for job in self.scheduler.get_jobs()
-                 if job.name == 'execute_audit' and
-                 job.args[0].uuid == audit.uuid][0].remove()
+            jobs = [job for job in self.scheduler.get_jobs()
+                    if job.name == 'execute_audit' and
+                    job.args[0].uuid == audit.uuid]
+            if jobs:
+                jobs[0].remove()
             return True
 
         return False
 
     def do_execute(self, audit, request_context):
-        # execute the strategy
-        solution = self.strategy_context.execute_strategy(
-            audit, request_context)
+        solution = super(ContinuousAuditHandler, self)\
+            .do_execute(audit, request_context)
 
         if audit.audit_type == objects.audit.AuditType.CONTINUOUS.value:
             a_plan_filters = {'audit_uuid': audit.uuid,
@@ -118,26 +120,54 @@ class ContinuousAuditHandler(base.AuditHandler):
                                name='execute_audit',
                                **trigger_args)
 
+    def check_audit_expired(self, audit):
+        current = datetime.datetime.utcnow()
+        # Note: if audit still didn't get into the timeframe,
+        #       skip it
+        if audit.start_time and audit.start_time > current:
+            return True
+        if audit.end_time and audit.end_time < current:
+            if audit.state != objects.audit.State.SUCCEEDED:
+                audit.state = objects.audit.State.SUCCEEDED
+                audit.save()
+            return True
+
+        return False
+
     def launch_audits_periodically(self):
         audit_context = context.RequestContext(is_admin=True)
         audit_filters = {
             'audit_type': objects.audit.AuditType.CONTINUOUS.value,
             'state__in': (objects.audit.State.PENDING,
-                          objects.audit.State.ONGOING,
-                          objects.audit.State.SUCCEEDED)
+                          objects.audit.State.ONGOING),
         }
-        audits = objects.Audit.list(
+        audit_filters['hostname'] = None
+        unscheduled_audits = objects.Audit.list(
             audit_context, filters=audit_filters, eager=True)
+        for audit in unscheduled_audits:
+            # If continuous audit doesn't have a hostname yet,
+            # Watcher will set current CONF.host value.
+            # TODO(alexchadin): Add scheduling of new continuous audits.
+            audit.hostname = CONF.host
+            audit.save()
         scheduler_job_args = [
             (job.args[0].uuid, job) for job
             in self.scheduler.get_jobs()
             if job.name == 'execute_audit']
         scheduler_jobs = dict(scheduler_job_args)
         # if audit isn't in active states, audit's job should be removed
+        jobs_to_remove = []
         for job in scheduler_jobs.values():
             if self._is_audit_inactive(job.args[0]):
-                scheduler_jobs.pop(job.args[0].uuid)
+                jobs_to_remove.append(job.args[0].uuid)
+        for audit_uuid in jobs_to_remove:
+            scheduler_jobs.pop(audit_uuid)
+        audit_filters['hostname'] = CONF.host
+        audits = objects.Audit.list(
+            audit_context, filters=audit_filters, eager=True)
         for audit in audits:
+            if self.check_audit_expired(audit):
+                continue
             existing_job = scheduler_jobs.get(audit.uuid, None)
             # if audit is not presented in scheduled audits yet,
             # just add a new audit job.
@@ -173,6 +203,7 @@ class ContinuousAuditHandler(base.AuditHandler):
                     audit.next_run_time = self._next_cron_time(audit)
                     self._add_job('date', audit, audit_context,
                                   run_date=audit.next_run_time)
+                audit.hostname = CONF.host
                 audit.save()
 
     def start(self):
