@@ -16,30 +16,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-"""
-*Good server consolidation strategy*
-
-Consolidation of VMs is essential to achieve energy optimization in cloud
-environments such as OpenStack. As VMs are spinned up and/or moved over time,
-it becomes necessary to migrate VMs among servers to lower the costs. However,
-migration of VMs introduces runtime overheads and consumes extra energy, thus
-a good server consolidation strategy should carefully plan for migration in
-order to both minimize energy consumption and comply to the various SLAs.
-
-This algorithm not only minimizes the overall number of used servers, but also
-minimizes the number of migrations.
-
-It has been developed only for tests. You must have at least 2 physical compute
-nodes to run it, so you can easily run it on DevStack. It assumes that live
-migration is possible on your OpenStack cluster.
-
-"""
 
 from oslo_config import cfg
 from oslo_log import log
 
 from watcher._i18n import _
-from watcher.common import exception
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -47,7 +28,25 @@ LOG = log.getLogger(__name__)
 
 
 class BasicConsolidation(base.ServerConsolidationBaseStrategy):
-    """Basic offline consolidation using live migration"""
+    """Good server consolidation strategy
+
+    Basic offline consolidation using live migration
+
+    Consolidation of VMs is essential to achieve energy optimization in cloud
+    environments such as OpenStack. As VMs are spinned up and/or moved over
+    time, it becomes necessary to migrate VMs among servers to lower the
+    costs. However, migration of VMs introduces runtime overheads and
+    consumes extra energy, thus a good server consolidation strategy should
+    carefully plan for migration in order to both minimize energy consumption
+    and comply to the various SLAs.
+
+    This algorithm not only minimizes the overall number of used servers,
+    but also minimizes the number of migrations.
+
+    It has been developed only for tests. You must have at least 2 physical
+    compute nodes to run it, so you can easily run it on DevStack. It assumes
+    that live migration is possible on your OpenStack cluster.
+    """
 
     HOST_CPU_USAGE_METRIC_NAME = 'compute.node.cpu.percent'
     INSTANCE_CPU_USAGE_METRIC_NAME = 'cpu_util'
@@ -109,6 +108,12 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
     def granularity(self):
         return self.input_parameters.get('granularity', 300)
 
+    @property
+    def aggregation_method(self):
+        return self.input_parameters.get(
+            'aggregation_method',
+            {"instance": 'mean', "node": 'mean'})
+
     @classmethod
     def get_display_name(cls):
         return _("Basic offline consolidation")
@@ -142,6 +147,26 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
                     "type": "number",
                     "default": 300
                 },
+                "aggregation_method": {
+                    "description": "Function used to aggregate multiple "
+                                   "measures into an aggregate. For example, "
+                                   "the min aggregation method will aggregate "
+                                   "the values of different measures to the "
+                                   "minimum value of all the measures in the "
+                                   "time range.",
+                    "type": "object",
+                    "properties": {
+                        "instance": {
+                            "type": "string",
+                            "default": 'mean'
+                        },
+                        "node": {
+                            "type": "string",
+                            "default": 'mean'
+                        },
+                    },
+                    "default": {"instance": 'mean', "node": 'mean'}
+                },
             },
         }
 
@@ -178,7 +203,7 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
         :param source_node: the current node of the virtual machine
         :param destination_node: the destination of the virtual machine
         :param instance_to_migrate: the instance / virtual machine
-        :return: True if the there is enough place otherwise false
+        :return: True if there is enough place otherwise false
         """
         if source_node == destination_node:
             return False
@@ -258,11 +283,13 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
     def get_node_cpu_usage(self, node):
         resource_id = "%s_%s" % (node.uuid, node.hostname)
         return self.datasource_backend.get_host_cpu_usage(
-            resource_id, self.period, 'mean', granularity=300)
+            resource_id, self.period, self.aggregation_method['node'],
+            granularity=self.granularity)
 
     def get_instance_cpu_usage(self, instance):
         return self.datasource_backend.get_instance_cpu_usage(
-            instance.uuid, self.period, 'mean', granularity=300)
+            instance.uuid, self.period, self.aggregation_method['instance'],
+            granularity=self.granularity)
 
     def calculate_score_node(self, node):
         """Calculate the score that represent the utilization level
@@ -277,10 +304,10 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
             resource_id = "%s_%s" % (node.uuid, node.hostname)
             LOG.error(
                 "No values returned by %(resource_id)s "
-                "for %(metric_name)s" % dict(
+                "for %(metric_name)s", dict(
                     resource_id=resource_id,
                     metric_name=self.METRIC_NAMES[
-                        self.config.datasource]['host_cpu_usage']))
+                        self.datasource_backend.NAME]['host_cpu_usage']))
             host_avg_cpu_util = 100
 
         total_cores_used = node.vcpus * (host_avg_cpu_util / 100.0)
@@ -297,10 +324,10 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
         if instance_cpu_utilization is None:
             LOG.error(
                 "No values returned by %(resource_id)s "
-                "for %(metric_name)s" % dict(
+                "for %(metric_name)s", dict(
                     resource_id=instance.uuid,
                     metric_name=self.METRIC_NAMES[
-                        self.config.datasource]['instance_cpu_usage']))
+                        self.datasource_backend.NAME]['instance_cpu_usage']))
             instance_cpu_utilization = 100
 
         total_cores_used = instance.vcpus * (instance_cpu_utilization / 100.0)
@@ -372,6 +399,11 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
                                  sorted_score):
         number_migrations = 0
         for mig_instance, __ in sorted_instances:
+            # skip exclude instance when migrating
+            if mig_instance.watcher_exclude:
+                LOG.debug("Instance is excluded by scope, "
+                          "skipped: %s", mig_instance.uuid)
+                continue
             for node_uuid, __ in sorted_score:
                 mig_source_node = self.compute_model.get_node_by_uuid(
                     node_to_release)
@@ -396,18 +428,7 @@ class BasicConsolidation(base.ServerConsolidationBaseStrategy):
             return unsuccessful_migration + 1
 
     def pre_execute(self):
-        LOG.info("Initializing Server Consolidation")
-
-        if not self.compute_model:
-            raise exception.ClusterStateNotDefined()
-
-        if len(self.get_available_compute_nodes()) == 0:
-            raise exception.ClusterEmpty()
-
-        if self.compute_model.stale:
-            raise exception.ClusterStateStale()
-
-        LOG.debug(self.compute_model.to_string())
+        self._pre_execute()
 
     def do_execute(self):
         unsuccessful_migration = 0
