@@ -11,13 +11,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-"""
-*Zone migration using instance and volume migration*
-
-This is zone migration strategy to migrate many instances and volumes
-efficiently with minimum downtime for hardware maintenance.
-
-"""
 
 from dateutil.parser import parse
 import six
@@ -28,7 +21,6 @@ from cinderclient.v2.volumes import Volume
 from novaclient.v2.servers import Server
 from watcher._i18n import _
 from watcher.common import cinder_helper
-from watcher.common import exception as wexc
 from watcher.common import nova_helper
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
@@ -48,7 +40,11 @@ IN_USE = "in-use"
 
 
 class ZoneMigration(base.ZoneMigrationBaseStrategy):
-    """Zone migration using instance and volume migration"""
+    """Zone migration using instance and volume migration
+
+    This is zone migration strategy to migrate many instances and volumes
+    efficiently with minimum downtime for hardware maintenance.
+    """
 
     def __init__(self, config, osc=None):
 
@@ -92,7 +88,7 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
                                 "type": "string"
                             },
                             "dst_node": {
-                                "description": "Compute node to which"
+                                "description": "Compute node to which "
                                                "instances migrate",
                                 "type": "string"
                             }
@@ -270,19 +266,7 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
                 cn.status in default_node_scope}
 
     def pre_execute(self):
-        """Pre-execution phase
-
-        This can be used to fetch some pre-requisites or data.
-        """
-        LOG.info("Initializing zone migration Strategy")
-
-        if len(self.get_available_compute_nodes()) == 0:
-            raise wexc.ComputeClusterEmpty()
-
-        if len(self.get_available_storage_nodes()) == 0:
-            raise wexc.StorageClusterEmpty()
-
-        LOG.debug(self.compute_model.to_string())
+        self._pre_execute()
         LOG.debug(self.storage_model.to_string())
 
     def do_execute(self):
@@ -312,10 +296,10 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
                     else:
                         self.instances_migration(targets, action_counter)
 
-        LOG.debug("action total: %s, pools: %s, nodes %s " % (
+        LOG.debug("action total: %s, pools: %s, nodes %s ",
                   action_counter.total_count,
                   action_counter.per_pool_count,
-                  action_counter.per_node_count))
+                  action_counter.per_node_count)
 
     def post_execute(self):
         """Post-execution phase
@@ -413,13 +397,13 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
             pool = getattr(volume, 'os-vol-host-attr:host')
             if action_counter.is_pool_max(pool):
                 LOG.debug("%s has objects to be migrated, but it has"
-                          " reached the limit of parallelization." % pool)
+                          " reached the limit of parallelization.", pool)
                 continue
 
             src_type = volume.volume_type
             dst_pool, dst_type = self.get_dst_pool_and_type(pool, src_type)
             LOG.debug(src_type)
-            LOG.debug("%s %s" % (dst_pool, dst_type))
+            LOG.debug("%s %s", dst_pool, dst_type)
 
             if self.is_available(volume):
                 if src_type == dst_type:
@@ -448,7 +432,7 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
 
             if action_counter.is_node_max(src_node):
                 LOG.debug("%s has objects to be migrated, but it has"
-                          " reached the limit of parallelization." % src_node)
+                          " reached the limit of parallelization.", src_node)
                 continue
 
             dst_node = self.get_dst_node(src_node)
@@ -538,8 +522,8 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
             return None
 
         return [i for i in self.nova.get_instance_list()
-                if getattr(i, 'OS-EXT-SRV-ATTR:host') in src_node_list
-                and self.compute_model.get_instance_by_uuid(i.id)]
+                if getattr(i, 'OS-EXT-SRV-ATTR:host') in src_node_list and
+                self.compute_model.get_instance_by_uuid(i.id)]
 
     def get_volumes(self):
         """Get migrate target volumes
@@ -550,8 +534,8 @@ class ZoneMigration(base.ZoneMigrationBaseStrategy):
         src_pool_list = self.get_src_pool_list()
 
         return [i for i in self.cinder.get_volume_list()
-                if getattr(i, 'os-vol-host-attr:host') in src_pool_list
-                and self.storage_model.get_volume_by_uuid(i.id)]
+                if getattr(i, 'os-vol-host-attr:host') in src_pool_list and
+                self.storage_model.get_volume_by_uuid(i.id)]
 
     def filtered_targets(self):
         """Filter targets
@@ -643,8 +627,8 @@ class ActionCounter(object):
         if not self.is_total_max() and not self.is_pool_max(pool):
             self.per_pool_count[pool] += 1
             self.total_count += 1
-            LOG.debug("total: %s, per_pool: %s" % (
-                      self.total_count, self.per_pool_count))
+            LOG.debug("total: %s, per_pool: %s",
+                      self.total_count, self.per_pool_count)
             return True
         return False
 
@@ -660,8 +644,8 @@ class ActionCounter(object):
         if not self.is_total_max() and not self.is_node_max(node):
             self.per_node_count[node] += 1
             self.total_count += 1
-            LOG.debug("total: %s, per_node: %s" % (
-                      self.total_count, self.per_node_count))
+            LOG.debug("total: %s, per_node: %s",
+                      self.total_count, self.per_node_count)
             return True
         return False
 
@@ -679,9 +663,9 @@ class ActionCounter(object):
         """
         if pool not in self.per_pool_count:
             self.per_pool_count[pool] = 0
-        LOG.debug("the number of parallel per pool %s is %s " %
-                  (pool, self.per_pool_count[pool]))
-        LOG.debug("per pool limit is %s" % self.per_pool_limit)
+        LOG.debug("the number of parallel per pool %s is %s ",
+                  pool, self.per_pool_count[pool])
+        LOG.debug("per pool limit is %s", self.per_pool_limit)
         return self.per_pool_count[pool] >= self.per_pool_limit
 
     def is_node_max(self, node):
@@ -724,7 +708,7 @@ class BaseFilter(object):
             for k, v in six.iteritems(targets):
                 if not self.is_allowed(k):
                     continue
-                LOG.debug("filter:%s with the key: %s" % (cond, k))
+                LOG.debug("filter:%s with the key: %s", cond, k)
                 targets[k] = self.exec_filter(v, cond)
 
         LOG.debug(targets)
@@ -778,7 +762,7 @@ class ProjectSortFilter(SortMovingToFrontFilter):
         """
 
         project_id = self.get_project_id(item)
-        LOG.debug("project_id: %s, sort_key: %s" % (project_id, sort_key))
+        LOG.debug("project_id: %s, sort_key: %s", project_id, sort_key)
         return project_id == sort_key
 
     def get_project_id(self, item):
@@ -812,7 +796,7 @@ class ComputeHostSortFilter(SortMovingToFrontFilter):
         """
 
         host = self.get_host(item)
-        LOG.debug("host: %s, sort_key: %s" % (host, sort_key))
+        LOG.debug("host: %s, sort_key: %s", host, sort_key)
         return host == sort_key
 
     def get_host(self, item):
@@ -840,7 +824,7 @@ class StorageHostSortFilter(SortMovingToFrontFilter):
         """
 
         host = self.get_host(item)
-        LOG.debug("host: %s, sort_key: %s" % (host, sort_key))
+        LOG.debug("host: %s, sort_key: %s", host, sort_key)
         return host == sort_key
 
     def get_host(self, item):
@@ -867,7 +851,7 @@ class ComputeSpecSortFilter(BaseFilter):
         result = items
 
         if sort_key not in self.accept_keys:
-            LOG.warning("Invalid key is specified: %s" % sort_key)
+            LOG.warning("Invalid key is specified: %s", sort_key)
         else:
             result = self.get_sorted_items(items, sort_key)
 
@@ -912,11 +896,11 @@ class ComputeSpecSortFilter(BaseFilter):
         :returns: memory size of item
         """
 
-        LOG.debug("item: %s, flavors: %s" % (item, flavors))
+        LOG.debug("item: %s, flavors: %s", item, flavors)
         for flavor in flavors:
-            LOG.debug("item.flavor: %s, flavor: %s" % (item.flavor, flavor))
+            LOG.debug("item.flavor: %s, flavor: %s", item.flavor, flavor)
             if item.flavor.get('id') == flavor.id:
-                LOG.debug("flavor.ram: %s" % flavor.ram)
+                LOG.debug("flavor.ram: %s", flavor.ram)
                 return flavor.ram
 
     def get_vcpu_num(self, item, flavors):
@@ -927,11 +911,11 @@ class ComputeSpecSortFilter(BaseFilter):
         :returns: vcpu number of item
         """
 
-        LOG.debug("item: %s, flavors: %s" % (item, flavors))
+        LOG.debug("item: %s, flavors: %s", item, flavors)
         for flavor in flavors:
-            LOG.debug("item.flavor: %s, flavor: %s" % (item.flavor, flavor))
+            LOG.debug("item.flavor: %s, flavor: %s", item.flavor, flavor)
             if item.flavor.get('id') == flavor.id:
-                LOG.debug("flavor.vcpus: %s" % flavor.vcpus)
+                LOG.debug("flavor.vcpus: %s", flavor.vcpus)
                 return flavor.vcpus
 
     def get_disk_size(self, item, flavors):
@@ -942,11 +926,11 @@ class ComputeSpecSortFilter(BaseFilter):
         :returns: disk size of item
         """
 
-        LOG.debug("item: %s, flavors: %s" % (item, flavors))
+        LOG.debug("item: %s, flavors: %s", item, flavors)
         for flavor in flavors:
-            LOG.debug("item.flavor: %s, flavor: %s" % (item.flavor, flavor))
+            LOG.debug("item.flavor: %s, flavor: %s", item.flavor, flavor)
             if item.flavor.get('id') == flavor.id:
-                LOG.debug("flavor.disk: %s" % flavor.disk)
+                LOG.debug("flavor.disk: %s", flavor.disk)
                 return flavor.disk
 
 
@@ -960,7 +944,7 @@ class StorageSpecSortFilter(BaseFilter):
         result = items
 
         if sort_key not in self.accept_keys:
-            LOG.warning("Invalid key is specified: %s" % sort_key)
+            LOG.warning("Invalid key is specified: %s", sort_key)
             return result
 
         if sort_key == 'created_at':
