@@ -48,6 +48,35 @@ class NotificationTestCase(base_test.TestCase):
 class TestReceiveNovaNotifications(NotificationTestCase):
 
     FAKE_METADATA = {'message_id': None, 'timestamp': None}
+    FAKE_NOTIFICATIONS = {
+        'instance.create.end': 'instance-create-end.json',
+        'instance.lock': 'instance-lock.json',
+        'instance.unlock': 'instance-unlock.json',
+        'instance.pause.end': 'instance-pause-end.json',
+        'instance.power_off.end': 'instance-power_off-end.json',
+        'instance.power_on.end': 'instance-power_on-end.json',
+        'instance.resize_confirm.end': 'instance-resize_confirm-end.json',
+        'instance.restore.end': 'instance-restore-end.json',
+        'instance.resume.end': 'instance-resume-end.json',
+        'instance.shelve.end': 'instance-shelve-end.json',
+        'instance.shutdown.end': 'instance-shutdown-end.json',
+        'instance.suspend.end': 'instance-suspend-end.json',
+        'instance.unpause.end': 'instance-unpause-end.json',
+        'instance.unrescue.end': 'instance-unrescue-end.json',
+        'instance.unshelve.end': 'instance-unshelve-end.json',
+        'instance.rebuild.end': 'instance-rebuild-end.json',
+        'instance.rescue.end': 'instance-rescue-end.json',
+        'instance.update': 'instance-update.json',
+        'instance.live_migration_force_complete.end':
+        'instance-live_migration_force_complete-end.json',
+        'instance.live_migration_post_dest.end':
+        'instance-live_migration_post_dest-end.json',
+        'instance.delete.end': 'instance-delete-end.json',
+        'instance.soft_delete.end': 'instance-soft_delete-end.json',
+        'service.create': 'service-create.json',
+        'service.delete': 'service-delete.json',
+        'service.update': 'service-update.json',
+        }
 
     def setUp(self):
         super(TestReceiveNovaNotifications, self).setUp()
@@ -61,57 +90,22 @@ class TestReceiveNovaNotifications(NotificationTestCase):
         self.m_heartbeat = p_heartbeat.start()
         self.addCleanup(p_heartbeat.stop)
 
-    @mock.patch.object(novanotification.ServiceUpdated, 'info')
-    def test_nova_receive_service_update(self, m_info):
-        message = self.load_message('service-update.json')
-        expected_message = message['payload']
-
+    @mock.patch.object(novanotification.VersionedNotification, 'info')
+    def test_receive_nova_notifications(self, m_info):
         de_service = watcher_service.Service(fake_managers.FakeManager)
-        incoming = mock.Mock(ctxt=self.context.to_dict(), message=message)
+        n_dicts = novanotification.VersionedNotification.notification_mapping
+        for n_type in n_dicts.keys():
+            n_json = self.FAKE_NOTIFICATIONS[n_type]
+            message = self.load_message(n_json)
+            expected_message = message['payload']
+            publisher_id = message['publisher_id']
 
-        de_service.notification_handler.dispatcher.dispatch(incoming)
-        m_info.assert_called_once_with(
-            self.context, 'nova-compute:host1', 'service.update',
-            expected_message, self.FAKE_METADATA)
+            incoming = mock.Mock(ctxt=self.context.to_dict(), message=message)
 
-    @mock.patch.object(novanotification.InstanceCreated, 'info')
-    def test_nova_receive_instance_create(self, m_info):
-        message = self.load_message('instance-create.json')
-        expected_message = message['payload']
-
-        de_service = watcher_service.Service(fake_managers.FakeManager)
-        incoming = mock.Mock(ctxt=self.context.to_dict(), message=message)
-
-        de_service.notification_handler.dispatcher.dispatch(incoming)
-        m_info.assert_called_once_with(
-            self.context, 'nova-compute:compute', 'instance.update',
-            expected_message, self.FAKE_METADATA)
-
-    @mock.patch.object(novanotification.InstanceUpdated, 'info')
-    def test_nova_receive_instance_update(self, m_info):
-        message = self.load_message('instance-update.json')
-        expected_message = message['payload']
-
-        de_service = watcher_service.Service(fake_managers.FakeManager)
-        incoming = mock.Mock(ctxt=self.context.to_dict(), message=message)
-
-        de_service.notification_handler.dispatcher.dispatch(incoming)
-        m_info.assert_called_once_with(
-            self.context, 'nova-compute:compute', 'instance.update',
-            expected_message, self.FAKE_METADATA)
-
-    @mock.patch.object(novanotification.InstanceDeletedEnd, 'info')
-    def test_nova_receive_instance_delete_end(self, m_info):
-        message = self.load_message('instance-delete-end.json')
-        expected_message = message['payload']
-
-        de_service = watcher_service.Service(fake_managers.FakeManager)
-        incoming = mock.Mock(ctxt=self.context.to_dict(), message=message)
-
-        de_service.notification_handler.dispatcher.dispatch(incoming)
-        m_info.assert_called_once_with(
-            self.context, 'nova-compute:compute', 'instance.delete.end',
-            expected_message, self.FAKE_METADATA)
+            de_service.notification_handler.dispatcher.dispatch(incoming)
+            m_info.assert_called_with(
+                self.context, publisher_id, n_type,
+                expected_message, self.FAKE_METADATA)
 
 
 class TestNovaNotifications(NotificationTestCase):
@@ -126,7 +120,7 @@ class TestNovaNotifications(NotificationTestCase):
     def test_nova_service_update(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.ServiceUpdated(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         node0_uuid = 'Node_0'
         node0 = compute_model.get_node_by_uuid(node0_uuid)
@@ -163,15 +157,81 @@ class TestNovaNotifications(NotificationTestCase):
         self.assertEqual(element.ServiceState.ONLINE.value, node0.state)
         self.assertEqual(element.ServiceState.ENABLED.value, node0.status)
 
+    @mock.patch.object(nova_helper, "NovaHelper")
+    def test_nova_service_create(self, m_nova_helper_cls):
+        m_get_compute_node_by_hostname = mock.Mock(
+            side_effect=lambda uuid: mock.Mock(
+                name='m_get_compute_node_by_hostname',
+                id=3,
+                hypervisor_hostname="host2",
+                state='up',
+                status='enabled',
+                uuid=uuid,
+                memory_mb=7777,
+                vcpus=42,
+                free_disk_gb=974,
+                local_gb=1337))
+        m_nova_helper_cls.return_value = mock.Mock(
+            get_compute_node_by_hostname=m_get_compute_node_by_hostname,
+            name='m_nova_helper')
+
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        new_node_uuid = 'host2'
+
+        self.assertRaises(
+            exception.ComputeNodeNotFound,
+            compute_model.get_node_by_uuid, new_node_uuid)
+
+        message = self.load_message('service-create.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        new_node = compute_model.get_node_by_uuid(new_node_uuid)
+        self.assertEqual('host2', new_node.hostname)
+        self.assertEqual(element.ServiceState.ONLINE.value, new_node.state)
+        self.assertEqual(element.ServiceState.ENABLED.value, new_node.status)
+
+    def test_nova_service_delete(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        node0_uuid = 'Node_0'
+
+        # Before
+        self.assertTrue(compute_model.get_node_by_uuid(node0_uuid))
+
+        message = self.load_message('service-delete.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        # After
+        self.assertRaises(
+            exception.ComputeNodeNotFound,
+            compute_model.get_node_by_uuid, node0_uuid)
+
     def test_nova_instance_update(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.InstanceUpdated(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
 
-        message = self.load_message('scenario3_instance-update.json')
+        message = self.load_message('instance-update.json')
 
         self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
 
@@ -205,7 +265,7 @@ class TestNovaNotifications(NotificationTestCase):
             name='m_nova_helper')
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.InstanceUpdated(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = '9966d6bd-a45c-4e1c-9d57-3054899a3ec7'
 
@@ -244,7 +304,7 @@ class TestNovaNotifications(NotificationTestCase):
 
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.InstanceUpdated(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = '9966d6bd-a45c-4e1c-9d57-3054899a3ec7'
 
@@ -275,7 +335,7 @@ class TestNovaNotifications(NotificationTestCase):
     def test_nova_instance_create(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.InstanceCreated(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = 'c03c0bf9-f46e-4e4f-93f1-817568567ee2'
 
@@ -283,7 +343,7 @@ class TestNovaNotifications(NotificationTestCase):
             exception.InstanceNotFound,
             compute_model.get_instance_by_uuid, instance0_uuid)
 
-        message = self.load_message('scenario3_instance-create.json')
+        message = self.load_message('instance-create-end.json')
         handler.info(
             ctxt=self.context,
             publisher_id=message['publisher_id'],
@@ -302,14 +362,14 @@ class TestNovaNotifications(NotificationTestCase):
     def test_nova_instance_delete_end(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.InstanceDeletedEnd(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
 
         # Before
         self.assertTrue(compute_model.get_instance_by_uuid(instance0_uuid))
 
-        message = self.load_message('scenario3_instance-delete-end.json')
+        message = self.load_message('instance-delete-end.json')
         handler.info(
             ctxt=self.context,
             publisher_id=message['publisher_id'],
@@ -323,169 +383,40 @@ class TestNovaNotifications(NotificationTestCase):
             exception.InstanceNotFound,
             compute_model.get_instance_by_uuid, instance0_uuid)
 
-
-class TestLegacyNovaNotifications(NotificationTestCase):
-
-    FAKE_METADATA = {'message_id': None, 'timestamp': None}
-
-    def setUp(self):
-        super(TestLegacyNovaNotifications, self).setUp()
-        # fake cluster
-        self.fake_cdmc = faker_cluster_state.FakerModelCollector()
-
-    def test_legacy_instance_created_end(self):
+    def test_nova_instance_soft_delete_end(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceCreatedEnd(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
-        instance0_uuid = 'c03c0bf9-f46e-4e4f-93f1-817568567ee2'
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+
+        # Before
+        self.assertTrue(compute_model.get_instance_by_uuid(instance0_uuid))
+
+        message = self.load_message('instance-soft_delete-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        # After
         self.assertRaises(
             exception.InstanceNotFound,
             compute_model.get_instance_by_uuid, instance0_uuid)
 
-        message = self.load_message(
-            'scenario3_legacy_instance-create-end.json')
-
-        handler.info(
-            ctxt=self.context,
-            publisher_id=message['publisher_id'],
-            event_type=message['event_type'],
-            payload=message['payload'],
-            metadata=self.FAKE_METADATA,
-        )
-
-        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-
-        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
-        self.assertEqual(1, instance0.vcpus)
-        self.assertEqual(1, instance0.disk_capacity)
-        self.assertEqual(512, instance0.memory)
-
-    def test_legacy_instance_updated(self):
+    def test_live_migrated_force_end(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceUpdated(self.fake_cdmc)
-
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
         instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-
-        message = self.load_message('scenario3_legacy_instance-update.json')
-
-        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
-
-        handler.info(
-            ctxt=self.context,
-            publisher_id=message['publisher_id'],
-            event_type=message['event_type'],
-            payload=message['payload'],
-            metadata=self.FAKE_METADATA,
-        )
-
-        self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-
-    @mock.patch.object(nova_helper, "NovaHelper")
-    def test_legacy_instance_update_node_notfound_still_creates(
-            self, m_nova_helper_cls):
-        m_get_compute_node_by_hostname = mock.Mock(
-            side_effect=lambda uuid: mock.Mock(
-                name='m_get_compute_node_by_hostname',
-                id=3,
-                uuid=uuid,
-                hypervisor_hostname="Node_2",
-                state='up',
-                status='enabled',
-                memory_mb=7777,
-                vcpus=42,
-                free_disk_gb=974,
-                local_gb=1337))
-        m_nova_helper_cls.return_value = mock.Mock(
-            get_compute_node_by_hostname=m_get_compute_node_by_hostname,
-            name='m_nova_helper')
-
-        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
-        self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceUpdated(self.fake_cdmc)
-
-        instance0_uuid = '9966d6bd-a45c-4e1c-9d57-3054899a3ec7'
-
-        message = self.load_message(
-            'scenario3_notfound_legacy_instance-update.json')
-
-        handler.info(
-            ctxt=self.context,
-            publisher_id=message['publisher_id'],
-            event_type=message['event_type'],
-            payload=message['payload'],
-            metadata=self.FAKE_METADATA,
-        )
-
-        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-
-        self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, instance0.vcpus)
-        self.assertEqual(1, instance0.disk)
-        self.assertEqual(1, instance0.disk_capacity)
-        self.assertEqual(512, instance0.memory)
-
-        m_get_compute_node_by_hostname.assert_any_call('Node_2')
-        node_2 = compute_model.get_node_by_uuid('Node_2')
-        self.assertEqual(7777, node_2.memory)
-        self.assertEqual(42, node_2.vcpus)
-        self.assertEqual(974, node_2.disk)
-        self.assertEqual(1337, node_2.disk_capacity)
-
-    @mock.patch.object(nova_helper, "NovaHelper")
-    def test_legacy_instance_update_node_notfound_set_unmapped(
-            self, m_nova_helper_cls):
-        m_get_compute_node_by_hostname = mock.Mock(
-            side_effect=exception.ComputeNodeNotFound)
-        m_nova_helper_cls.return_value = mock.Mock(
-            get_compute_node_by_hostname=m_get_compute_node_by_hostname,
-            name='m_nova_helper')
-
-        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
-        self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceUpdated(self.fake_cdmc)
-
-        instance0_uuid = '9966d6bd-a45c-4e1c-9d57-3054899a3ec7'
-
-        message = self.load_message(
-            'scenario3_notfound_legacy_instance-update.json')
-
-        handler.info(
-            ctxt=self.context,
-            publisher_id=message['publisher_id'],
-            event_type=message['event_type'],
-            payload=message['payload'],
-            metadata=self.FAKE_METADATA,
-        )
-
-        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-
-        self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, instance0.vcpus)
-        self.assertEqual(1, instance0.disk)
-        self.assertEqual(1, instance0.disk_capacity)
-        self.assertEqual(512, instance0.memory)
-
-        m_get_compute_node_by_hostname.assert_any_call('Node_2')
-        self.assertRaises(
-            exception.ComputeNodeNotFound,
-            compute_model.get_node_by_uuid, 'Node_2')
-
-    def test_legacy_live_migrated_end(self):
-        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
-        self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyLiveMigratedEnd(self.fake_cdmc)
-
-        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
-        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-
         node = compute_model.get_node_by_instance_uuid(instance0_uuid)
         self.assertEqual('Node_0', node.uuid)
-
         message = self.load_message(
-            'scenario3_legacy_livemigration-post-dest-end.json')
+            'instance-live_migration_force_complete-end.json')
         handler.info(
             ctxt=self.context,
             publisher_id=message['publisher_id'],
@@ -497,18 +428,39 @@ class TestLegacyNovaNotifications(NotificationTestCase):
         self.assertEqual('Node_1', node.uuid)
         self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
 
-    def test_legacy_instance_deleted_end(self):
+    def test_live_migrated_end(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceDeletedEnd(self.fake_cdmc)
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_0', node.uuid)
+        message = self.load_message(
+            'instance-live_migration_post_dest-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_1', node.uuid)
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_lock(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
 
         instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
 
-        # Before
-        self.assertTrue(compute_model.get_instance_by_uuid(instance0_uuid))
+        message = self.load_message('instance-lock.json')
+
+        self.assertFalse(instance0.locked)
 
-        message = self.load_message(
-            'scenario3_legacy_instance-delete-end.json')
         handler.info(
             ctxt=self.context,
             publisher_id=message['publisher_id'],
@@ -517,7 +469,277 @@ class TestLegacyNovaNotifications(NotificationTestCase):
             metadata=self.FAKE_METADATA,
         )
 
-        # After
-        self.assertRaises(
-            exception.InstanceNotFound,
-            compute_model.get_instance_by_uuid, instance0_uuid)
+        self.assertTrue(instance0.locked)
+
+        message = self.load_message('instance-unlock.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertFalse(instance0.locked)
+
+    def test_nova_instance_pause(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-pause-end.json')
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
+
+        message = self.load_message('instance-unpause-end.json')
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_power_on_off(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-power_off-end.json')
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.STOPPED.value, instance0.state)
+
+        message = self.load_message('instance-power_on-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_instance_rebuild_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_0', node.uuid)
+        message = self.load_message('instance-rebuild-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_1', node.uuid)
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_rescue(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-rescue-end.json')
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.RESCUED.value, instance0.state)
+
+        message = self.load_message('instance-unrescue-end.json')
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_instance_resize_confirm_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_0', node.uuid)
+        message = self.load_message(
+            'instance-resize_confirm-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_1', node.uuid)
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_restore_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-restore-end.json')
+        instance0.state = element.InstanceState.ERROR.value
+        self.assertEqual(element.InstanceState.ERROR.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_resume_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-resume-end.json')
+        instance0.state = element.InstanceState.ERROR.value
+        self.assertEqual(element.InstanceState.ERROR.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_shelve(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-shelve-end.json')
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.SHELVED.value, instance0.state)
+
+        message = self.load_message('instance-unshelve-end.json')
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_nova_instance_shutdown_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-shutdown-end.json')
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(element.InstanceState.STOPPED.value, instance0.state)
+
+    def test_nova_instance_suspend_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.VersionedNotification(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        message = self.load_message('instance-suspend-end.json')
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual(
+            element.InstanceState.SUSPENDED.value, instance0.state)
