@@ -18,18 +18,17 @@
 # limitations under the License.
 #
 
-import datetime
 import mock
 
-from watcher.common import exception
 from watcher.decision_engine.model import element
-from watcher.decision_engine.model import model_root
+from watcher.decision_engine.solution.base import BaseSolution
 from watcher.decision_engine.strategy import strategies
-from watcher.tests import base
 from watcher.tests.decision_engine.model import faker_cluster_and_metrics
+from watcher.tests.decision_engine.strategy.strategies.test_base \
+    import TestBaseStrategy
 
 
-class TestVMWorkloadConsolidation(base.TestCase):
+class TestVMWorkloadConsolidation(TestBaseStrategy):
 
     scenarios = [
         ("Ceilometer",
@@ -46,50 +45,32 @@ class TestVMWorkloadConsolidation(base.TestCase):
         super(TestVMWorkloadConsolidation, self).setUp()
 
         # fake cluster
-        self.fake_cluster = faker_cluster_and_metrics.FakerModelCollector()
-
-        p_model = mock.patch.object(
-            strategies.VMWorkloadConsolidation, "compute_model",
-            new_callable=mock.PropertyMock)
-        self.m_model = p_model.start()
-        self.addCleanup(p_model.stop)
+        self.fake_c_cluster = faker_cluster_and_metrics.FakerModelCollector()
 
         p_datasource = mock.patch.object(
-            strategies.VMWorkloadConsolidation, self.datasource,
+            strategies.VMWorkloadConsolidation, 'datasource_backend',
             new_callable=mock.PropertyMock)
         self.m_datasource = p_datasource.start()
         self.addCleanup(p_datasource.stop)
 
-        p_audit_scope = mock.patch.object(
-            strategies.VMWorkloadConsolidation, "audit_scope",
-            new_callable=mock.PropertyMock
-        )
-        self.m_audit_scope = p_audit_scope.start()
-        self.addCleanup(p_audit_scope.stop)
-
-        self.m_audit_scope.return_value = mock.Mock()
-
         # fake metrics
         self.fake_metrics = self.fake_datasource_cls(
-            self.m_model.return_value)
+            self.m_c_model.return_value)
 
-        self.m_model.return_value = model_root.ModelRoot()
         self.m_datasource.return_value = mock.Mock(
-            statistic_aggregation=self.fake_metrics.mock_get_statistics)
+            get_instance_cpu_usage=(
+                self.fake_metrics.get_instance_cpu_util),
+            get_instance_memory_usage=(
+                self.fake_metrics.get_instance_ram_util),
+            get_instance_root_disk_allocated=(
+                self.fake_metrics.get_instance_disk_root_size),
+            )
         self.strategy = strategies.VMWorkloadConsolidation(
-            config=mock.Mock(datasource=self.datasource))
-
-    def test_exception_stale_cdm(self):
-        self.fake_cluster.set_cluster_data_model_as_stale()
-        self.m_model.return_value = self.fake_cluster.cluster_data_model
-
-        self.assertRaises(
-            exception.ClusterStateNotDefined,
-            self.strategy.execute)
+            config=mock.Mock(datasources=self.datasource))
 
     def test_get_instance_utilization(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         instance_0 = model.get_instance_by_uuid("INSTANCE_0")
         instance_util = dict(cpu=1.0, ram=1, disk=10)
@@ -98,8 +79,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
             self.strategy.get_instance_utilization(instance_0))
 
     def test_get_node_utilization(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         node_0 = model.get_node_by_uuid("Node_0")
         node_util = dict(cpu=1.0, ram=1, disk=10)
@@ -108,16 +89,16 @@ class TestVMWorkloadConsolidation(base.TestCase):
             self.strategy.get_node_utilization(node_0))
 
     def test_get_node_capacity(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         node_0 = model.get_node_by_uuid("Node_0")
         node_util = dict(cpu=40, ram=64, disk=250)
         self.assertEqual(node_util, self.strategy.get_node_capacity(node_0))
 
     def test_get_relative_node_utilization(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         node = model.get_node_by_uuid('Node_0')
         rhu = self.strategy.get_relative_node_utilization(node)
@@ -125,16 +106,16 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected_rhu, rhu)
 
     def test_get_relative_cluster_utilization(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         cru = self.strategy.get_relative_cluster_utilization()
         expected_cru = {'cpu': 0.05, 'disk': 0.05, 'ram': 0.0234375}
         self.assertEqual(expected_cru, cru)
 
     def test_add_migration_with_active_state(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         n2 = model.get_node_by_uuid('Node_1')
@@ -150,8 +131,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions[0])
 
     def test_add_migration_with_paused_state(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         n2 = model.get_node_by_uuid('Node_1')
@@ -172,8 +153,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions[0])
 
     def test_is_overloaded(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         cc = {'cpu': 1.0, 'ram': 1.0, 'disk': 1.0}
@@ -189,8 +170,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertTrue(res)
 
     def test_instance_fits(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n = model.get_node_by_uuid('Node_1')
         instance0 = model.get_instance_by_uuid('INSTANCE_0')
@@ -203,8 +184,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertFalse(res)
 
     def test_add_action_enable_compute_node(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n = model.get_node_by_uuid('Node_0')
         self.strategy.add_action_enable_compute_node(n)
@@ -214,8 +195,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions)
 
     def test_add_action_disable_node(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n = model.get_node_by_uuid('Node_0')
         self.strategy.add_action_disable_node(n)
@@ -227,8 +208,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions)
 
     def test_disable_unused_nodes(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         n2 = model.get_node_by_uuid('Node_1')
@@ -250,8 +231,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions[1])
 
     def test_offload_phase(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         cc = {'cpu': 1.0, 'ram': 1.0, 'disk': 1.0}
         self.strategy.offload_phase(cc)
@@ -259,8 +240,8 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions)
 
     def test_consolidation_phase(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_1()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         n2 = model.get_node_by_uuid('Node_1')
@@ -275,16 +256,22 @@ class TestVMWorkloadConsolidation(base.TestCase):
         self.assertEqual(expected, self.strategy.solution.actions)
 
     def test_strategy(self):
-        model = self.fake_cluster.generate_scenario_2()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_2()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
+
+        result = self.strategy.pre_execute()
+        self.assertIsNone(result)
+
         n1 = model.get_node_by_uuid('Node_0')
-        cc = {'cpu': 1.0, 'ram': 1.0, 'disk': 1.0}
-        self.strategy.offload_phase(cc)
-        self.strategy.consolidation_phase(cc)
-        self.strategy.optimize_solution()
+        self.strategy.get_relative_cluster_utilization = mock.MagicMock()
+        self.strategy.do_execute()
         n2 = self.strategy.solution.actions[0][
             'input_parameters']['destination_node']
+        n3 = self.strategy.solution.actions[2][
+            'input_parameters']['resource_id']
+        n4 = self.strategy.solution.actions[3][
+            'input_parameters']['resource_id']
         expected = [{'action_type': 'migrate',
                      'input_parameters': {'destination_node': n2,
                                           'source_node': n1.uuid,
@@ -294,13 +281,35 @@ class TestVMWorkloadConsolidation(base.TestCase):
                      'input_parameters': {'destination_node': n2,
                                           'source_node': n1.uuid,
                                           'migration_type': 'live',
-                                          'resource_id': 'INSTANCE_1'}}]
-
+                                          'resource_id': 'INSTANCE_1'}},
+                    {'action_type': 'change_nova_service_state',
+                     'input_parameters': {'state': 'disabled',
+                                          'disabled_reason':
+                                          'watcher_disabled',
+                                          'resource_id': n3}},
+                    {'action_type': 'change_nova_service_state',
+                     'input_parameters': {'state': 'disabled',
+                                          'disabled_reason':
+                                          'watcher_disabled',
+                                          'resource_id': n4}}]
         self.assertEqual(expected, self.strategy.solution.actions)
 
+        compute_nodes_count = len(self.strategy.get_available_compute_nodes())
+        number_of_released_nodes = self.strategy.number_of_released_nodes
+        number_of_migrations = self.strategy.number_of_migrations
+        with mock.patch.object(
+            BaseSolution, 'set_efficacy_indicators'
+        ) as mock_set_efficacy_indicators:
+            result = self.strategy.post_execute()
+            mock_set_efficacy_indicators.assert_called_once_with(
+                compute_nodes_count=compute_nodes_count,
+                released_compute_nodes_count=number_of_released_nodes,
+                instance_migrations_count=number_of_migrations
+            )
+
     def test_strategy2(self):
-        model = self.fake_cluster.generate_scenario_3()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_3()
+        self.m_c_model.return_value = model
         self.fake_metrics.model = model
         n1 = model.get_node_by_uuid('Node_0')
         n2 = model.get_node_by_uuid('Node_1')
@@ -333,41 +342,3 @@ class TestVMWorkloadConsolidation(base.TestCase):
         del expected[3]
         del expected[1]
         self.assertEqual(expected, self.strategy.solution.actions)
-
-    def test_periods(self):
-        model = self.fake_cluster.generate_scenario_1()
-        self.m_model.return_value = model
-        p_ceilometer = mock.patch.object(
-            strategies.VMWorkloadConsolidation, "ceilometer")
-        m_ceilometer = p_ceilometer.start()
-        self.addCleanup(p_ceilometer.stop)
-        p_gnocchi = mock.patch.object(
-            strategies.VMWorkloadConsolidation, "gnocchi")
-        m_gnocchi = p_gnocchi.start()
-        self.addCleanup(p_gnocchi.stop)
-        datetime_patcher = mock.patch.object(
-            datetime, 'datetime',
-            mock.Mock(wraps=datetime.datetime)
-        )
-        mocked_datetime = datetime_patcher.start()
-        mocked_datetime.utcnow.return_value = datetime.datetime(
-            2017, 3, 19, 18, 53, 11, 657417)
-        self.addCleanup(datetime_patcher.stop)
-        m_ceilometer.return_value = mock.Mock(
-            statistic_aggregation=self.fake_metrics.mock_get_statistics)
-        m_gnocchi.return_value = mock.Mock(
-            statistic_aggregation=self.fake_metrics.mock_get_statistics)
-        instance0 = model.get_instance_by_uuid("INSTANCE_0")
-        self.strategy.get_instance_utilization(instance0)
-        if self.strategy.config.datasource == "ceilometer":
-            m_ceilometer.statistic_aggregation.assert_any_call(
-                aggregate='avg', meter_name='disk.root.size',
-                period=3600, resource_id=instance0.uuid)
-        elif self.strategy.config.datasource == "gnocchi":
-            stop_time = datetime.datetime.utcnow()
-            start_time = stop_time - datetime.timedelta(
-                seconds=int('3600'))
-            m_gnocchi.statistic_aggregation.assert_called_with(
-                resource_id=instance0.uuid, metric='disk.root.size',
-                granularity=300, start_time=start_time, stop_time=stop_time,
-                aggregation='mean')
