@@ -20,17 +20,15 @@ import collections
 import mock
 
 from watcher.applier.loading import default
-from watcher.common import exception
 from watcher.common import utils
-from watcher.decision_engine.model import model_root
 from watcher.decision_engine.strategy import strategies
-from watcher.tests import base
 from watcher.tests.decision_engine.model import ceilometer_metrics
-from watcher.tests.decision_engine.model import faker_cluster_state
 from watcher.tests.decision_engine.model import gnocchi_metrics
+from watcher.tests.decision_engine.strategy.strategies.test_base \
+    import TestBaseStrategy
 
 
-class TestWorkloadBalance(base.TestCase):
+class TestWorkloadBalance(TestBaseStrategy):
 
     scenarios = [
         ("Ceilometer",
@@ -45,14 +43,6 @@ class TestWorkloadBalance(base.TestCase):
         super(TestWorkloadBalance, self).setUp()
         # fake metrics
         self.fake_metrics = self.fake_datasource_cls()
-        # fake cluster
-        self.fake_cluster = faker_cluster_state.FakerModelCollector()
-
-        p_model = mock.patch.object(
-            strategies.WorkloadBalance, "compute_model",
-            new_callable=mock.PropertyMock)
-        self.m_model = p_model.start()
-        self.addCleanup(p_model.stop)
 
         p_datasource = mock.patch.object(
             strategies.WorkloadBalance, "datasource_backend",
@@ -60,14 +50,6 @@ class TestWorkloadBalance(base.TestCase):
         self.m_datasource = p_datasource.start()
         self.addCleanup(p_datasource.stop)
 
-        p_audit_scope = mock.patch.object(
-            strategies.WorkloadBalance, "audit_scope",
-            new_callable=mock.PropertyMock
-        )
-        self.m_audit_scope = p_audit_scope.start()
-        self.addCleanup(p_audit_scope.stop)
-
-        self.m_audit_scope.return_value = mock.Mock()
         self.m_datasource.return_value = mock.Mock(
             statistic_aggregation=self.fake_metrics.mock_get_statistics_wb)
         self.strategy = strategies.WorkloadBalance(
@@ -83,8 +65,8 @@ class TestWorkloadBalance(base.TestCase):
         self.strategy._granularity = 300
 
     def test_calc_used_resource(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         node = model.get_node_by_uuid('Node_0')
         cores_used, mem_used, disk_used = (
             self.strategy.calculate_used_resource(node))
@@ -92,8 +74,8 @@ class TestWorkloadBalance(base.TestCase):
         self.assertEqual((cores_used, mem_used, disk_used), (20, 64, 40))
 
     def test_group_hosts_by_cpu_util(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         self.strategy.threshold = 30
         n1, n2, avg, w_map = self.strategy.group_hosts_by_cpu_or_ram_util()
         self.assertEqual(n1[0]['node'].uuid, 'Node_0')
@@ -101,8 +83,8 @@ class TestWorkloadBalance(base.TestCase):
         self.assertEqual(avg, 8.0)
 
     def test_group_hosts_by_ram_util(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         self.strategy._meter = "memory.resident"
         self.strategy.threshold = 30
         n1, n2, avg, w_map = self.strategy.group_hosts_by_cpu_or_ram_util()
@@ -111,8 +93,8 @@ class TestWorkloadBalance(base.TestCase):
         self.assertEqual(avg, 33.0)
 
     def test_choose_instance_to_migrate(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         n1, n2, avg, w_map = self.strategy.group_hosts_by_cpu_or_ram_util()
         instance_to_mig = self.strategy.choose_instance_to_migrate(
             n1, avg, w_map)
@@ -121,8 +103,8 @@ class TestWorkloadBalance(base.TestCase):
                          "73b09e16-35b7-4922-804e-e8f5d9b740fc")
 
     def test_choose_instance_notfound(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         n1, n2, avg, w_map = self.strategy.group_hosts_by_cpu_or_ram_util()
         instances = model.get_all_instances()
         [model.remove_instance(inst) for inst in instances.values()]
@@ -131,8 +113,8 @@ class TestWorkloadBalance(base.TestCase):
         self.assertIsNone(instance_to_mig)
 
     def test_filter_destination_hosts(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         self.strategy.datasource = mock.MagicMock(
             statistic_aggregation=self.fake_metrics.mock_get_statistics_wb)
         n1, n2, avg, w_map = self.strategy.group_hosts_by_cpu_or_ram_util()
@@ -143,38 +125,16 @@ class TestWorkloadBalance(base.TestCase):
         self.assertEqual(len(dest_hosts), 1)
         self.assertEqual(dest_hosts[0]['node'].uuid, 'Node_1')
 
-    def test_exception_model(self):
-        self.m_model.return_value = None
-        self.assertRaises(
-            exception.ClusterStateNotDefined, self.strategy.execute)
-
-    def test_exception_cluster_empty(self):
-        model = model_root.ModelRoot()
-        self.m_model.return_value = model
-        self.assertRaises(exception.ClusterEmpty, self.strategy.execute)
-
-    def test_exception_stale_cdm(self):
-        self.fake_cluster.set_cluster_data_model_as_stale()
-        self.m_model.return_value = self.fake_cluster.cluster_data_model
-
-        self.assertRaises(
-            exception.ClusterStateNotDefined,
-            self.strategy.execute)
-
-    def test_execute_cluster_empty(self):
-        model = model_root.ModelRoot()
-        self.m_model.return_value = model
-        self.assertRaises(exception.ClusterEmpty, self.strategy.execute)
-
     def test_execute_no_workload(self):
-        model = self.fake_cluster.generate_scenario_4_with_1_node_no_instance()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.\
+            generate_scenario_4_with_1_node_no_instance()
+        self.m_c_model.return_value = model
         solution = self.strategy.execute()
         self.assertEqual([], solution.actions)
 
     def test_execute(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         solution = self.strategy.execute()
         actions_counter = collections.Counter(
             [action.get('action_type') for action in solution.actions])
@@ -183,8 +143,8 @@ class TestWorkloadBalance(base.TestCase):
         self.assertEqual(num_migrations, 1)
 
     def test_check_parameters(self):
-        model = self.fake_cluster.generate_scenario_6_with_2_nodes()
-        self.m_model.return_value = model
+        model = self.fake_c_cluster.generate_scenario_6_with_2_nodes()
+        self.m_c_model.return_value = model
         solution = self.strategy.execute()
         loader = default.DefaultActionLoader()
         for action in solution.actions:
